@@ -3,16 +3,16 @@
 // Purpose:     The class to store a DVD Structure (Titles/Menus)
 // Author:      Alex Thuering
 // Created:	29.01.2003
-// RCS-ID:      $Id: DVD.cpp,v 1.43 2008/05/18 19:17:02 ntalex Exp $
+// RCS-ID:      $Id: DVD.cpp,v 1.45 2008/11/09 19:59:31 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
 
 #include "DVD.h"
 #include "Config.h"
+#include <wx/file.h>
 #include <wxSVGXML/svgxmlhelpr.h>
 #include <wxVillaLib/utils.h>
-#include <sys/stat.h>
 #include <wxSVG/mediadec_ffmpeg.h>
 
 #define DATA_FILE(fname) wxFindDataFile(_T("data") + wxString(wxFILE_SEP_PATH) + fname)
@@ -129,16 +129,37 @@ Stream::Stream(StreamType type, wxString codecName) {
 	m_sourceChannelNumber = -1;
 }
 
+VideoFormat Stream::GetSourceVideoFormat() {
+	if (m_sourceVideoSize.GetWidth() == 720 && m_sourceVideoSize.GetHeight() == 576)
+		return vfPAL;
+	else if (m_sourceVideoSize.GetWidth() == 720 && m_sourceVideoSize.GetHeight() == 480)
+		return vfNTSC;
+	return vfNONE;
+}
+
+AudioFormat Stream::GetSourceAudioFormat() {
+	if (m_sourceCodecName == wxT("mp2"))
+		return afMP2;
+	else if (m_sourceCodecName == wxT("liba52") || m_sourceCodecName == wxT("ac3")
+			|| m_sourceCodecName == wxT("ac-3"))
+		return afAC3;
+	return afNONE;
+}
+
 wxString Stream::GetSourceFormat() {
 	wxString result = m_sourceCodecName;
+	if (result == wxT("liba52") || m_sourceCodecName == wxT("ac-3"))
+		result = wxT("ac3");
+	else if (result == wxT("mpeg2video"))
+		result = wxT("mpeg2");
 	switch (m_type) {
 	case stVIDEO:
 		if (m_sourceVideoSize.IsFullySpecified()) {
 			result += wxString::Format(wxT(", %dx%d"),
 					m_sourceVideoSize.GetWidth(), m_sourceVideoSize.GetHeight());
-			if (m_sourceVideoSize.GetWidth() == 720 && m_sourceVideoSize.GetHeight() == 576)
+			if (GetSourceVideoFormat() == vfPAL)
 				result += wxT(" (PAL)");
-			else if (m_sourceVideoSize.GetWidth() == 720 && m_sourceVideoSize.GetHeight() == 480)
+			else if (GetSourceVideoFormat() == vfNTSC)
 				result += wxT(" (NTSC)");
 		}
 		break;
@@ -185,8 +206,10 @@ bool Vob::SetFilename(wxString value) {
 		if (stream->GetType() == stAUDIO) {
 			stream->SetSourceChannelNumber(ffmpeg.GetChannelNumber(stIdx));
 			stream->SetSourceSampleRate(ffmpeg.GetSampleRate(stIdx));
+			stream->SetSourceBitrate(ffmpeg.GetBitrate(stIdx));
 		} else if (stream->GetType() == stVIDEO) {
 			stream->SetSourceVideoSize(ffmpeg.GetVideoSize());
+			stream->SetSourceBitrate(ffmpeg.GetBitrate(stIdx));
 		} else
 			continue;
 		m_streams.Insert(stream, stIdx);
@@ -215,6 +238,8 @@ bool Vob::AddAudioFile(wxString filename) {
 		m_streams.Add(stream);
 	} else
 		m_streams.Add(new Stream(stAUDIO, wxT("unknown")));
+	if (GetFilename().length() == 0) // menu or slideshow
+		m_duration = ffmpeg.GetDuration();
 	return true;
 }
 
@@ -233,6 +258,7 @@ long Vob::GetChapterTime(int chapter) {
 		while (1) {
 			time *= 60;
 			if (timeStr.Find(wxT(':')) == wxNOT_FOUND) {
+				time *= 1000;
 				double t = 0;
 				if (timeStr.ToDouble(&t))
 					time += (int) (t*1000);
@@ -392,10 +418,7 @@ bool Vob::PutXML(wxSvgXmlNode* node) {
 }
 
 unsigned int Vob::GetFileSize(wxString filename) {
-	wxStructStat fInfo;
-	if (wxStat(filename, &fInfo) != 0)
-		return 0;
-	return fInfo.st_size/1024;
+	return wxFile(filename).Length()/1024;
 }
 
 int Vob::GetSize(bool generated) {
@@ -407,19 +430,57 @@ int Vob::GetSize(bool generated) {
 		return 0;
 	}
 
-	int size = 0;
-	if (GetMenu())
-		size += 250;
-	else if (GetSlideshow())
-		size += 150*GetSlideshow()->GetDuration()*GetSlideshow()->Count();
-	else {
-		if (GetFilename().length())
+	long size = 0;
+	int stIdx = 0;
+	if (GetMenu()) {
+		double duration = m_duration;
+		if (duration < 1)
+			duration = s_config.GetMenuFrameCount()/(GetMenu()->GetVideoFormat() == vfPAL ? 25 : 30);
+		size += (long)(duration*s_config.GetMenuVideoBitrate()/8/3);
+	} else if (GetSlideshow()) {
+		size += GetSlideshow()->GetDuration()*GetSlideshow()->Count()*s_config.GetSlideshowVideoBitrate()/8/3;
+	} else {
+		if (m_doNotTranscode) {
 			size += GetFileSize(GetFilename());
-		for (unsigned int i=0; i<GetAudioFilenames().Count(); i++)
-			size += GetFileSize(GetAudioFilenames()[i]);
-		for (unsigned int i=0; i<GetSubtitles().Count(); i++)
-			size += GetFileSize(GetSubtitles()[i]->GetFilename());
+		} else {
+			if (GetFilename().length()) {
+				int streamsCount = (int)m_streams.GetCount() - m_audioFilenames.GetCount();
+				bool copyVideo = false;
+				long audioSize = 0;
+				for (; stIdx < streamsCount; stIdx++) {
+					Stream* stream = m_streams[stIdx];
+					switch (stream->GetType()) {
+					case stVIDEO:
+						if (stream->GetVideoFormat() == vfCOPY)
+							copyVideo = true;
+						else
+							size += (long) (GetDuration()*s_config.GetVideoBitrate()/8);
+						break;
+					case stAUDIO:
+						int srcBitrate = stream->GetSourceBitrate() > 0 ? stream->GetSourceBitrate() : 64000;
+						audioSize += (long) (GetDuration()*srcBitrate/8/1024);
+						if (stream->GetAudioFormat() == afCOPY)
+							size += (long) (GetDuration()*srcBitrate/8/1024);
+						else if (stream->GetAudioFormat() != afNONE)
+							size += (long) (GetDuration()*s_config.GetAudioBitrate()/8);
+						break;
+					}
+				}
+				if (copyVideo)
+					size += GetFileSize(GetFilename()) - audioSize;
+			}
+		}
 	}
+	for (unsigned int i=0; i<GetAudioFilenames().Count(); i++) {
+		Stream* stream = m_streams[stIdx++];
+		size += GetFileSize(GetAudioFilenames()[i]);
+		if (stream->GetAudioFormat() == afCOPY)
+			size += GetFileSize(GetAudioFilenames()[i]);
+		else
+			size += (long) (GetDuration()*s_config.GetAudioBitrate()/8);
+	}
+	for (unsigned int i=0; i<GetSubtitles().Count(); i++)
+		size += GetFileSize(GetSubtitles()[i]->GetFilename());
 	return size;
 }
 
@@ -1141,8 +1202,8 @@ wxArrayString DVD::GetAudioFormatLabels(bool copy, bool none) {
 		formats.Add(_T("None"));
 	if (copy)
 		formats.Add(_T("Copy"));
-	formats.Add(_T("MP2"));
-	formats.Add(_T("AC3"));
+	formats.Add(_T("MP2 48 kHz"));
+	formats.Add(_T("AC3 48 kHz"));
 	return formats;
 }
 
