@@ -3,7 +3,7 @@
 // Purpose:     Main window
 // Author:      Alex Thuering
 // Created:     10.10.2003
-// RCS-ID:      $Id: MainWin.cpp,v 1.90 2008/08/24 14:02:00 ntalex Exp $
+// RCS-ID:      $Id: MainWin.cpp,v 1.97 2008/11/11 20:08:06 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -27,8 +27,6 @@
 
 #ifdef __WXMSW__
 #include <wx/msw/registry.h>
-#else
-#define wxDirDialogDefaultFolderStr _T("~")
 #endif
 
 #include "rc/new.png.h"
@@ -61,7 +59,8 @@ enum
   DIR_TREE_BT_ID = 2250,
   BACKGROUNS_BT_ID,
   BUTTONS_BT_ID,
-  THUMBMENU_BACKGROUND_ID
+  THUMBMENU_BACKGROUND_ID,
+  REFRESH_ID
 };
 
 BEGIN_EVENT_TABLE(MainWin, wxFrame)
@@ -96,6 +95,7 @@ BEGIN_EVENT_TABLE(MainWin, wxFrame)
   
   EVT_MENU(THUMBMENU_BACKGROUND_ID, MainWin::OnSetBackground)
   EVT_UPDATE_UI(THUMBMENU_BACKGROUND_ID, MainWin::OnUpdateUISetBackground)
+  EVT_MENU(REFRESH_ID, MainWin::OnRefresh)
   
   EVT_SIZE(MainWin::OnResize)
   EVT_SPLITTER_SASH_POS_CHANGED(SPLIITTER_TITLE_ID, MainWin::OnSplitterTitleSash)
@@ -106,40 +106,40 @@ MainWin::MainWin(): wxFrame(NULL, -1, _T(""), wxDefaultPosition, wxDefaultSize,
 		wxDEFAULT_FRAME_STYLE), m_dvd(NULL) {
     // begin wxGlade: MainWin::MainWin
     m_splitterTitle = new wxSplitterWindow(this, SPLIITTER_TITLE_ID, wxDefaultPosition, wxDefaultSize, wxCLIP_CHILDREN);
-    m_splitterDir = new wxSplitterWindow(m_splitterTitle, -1, wxDefaultPosition, wxDefaultSize, wxCLIP_CHILDREN);
-    m_splitterThumb = new wxSplitterWindow(m_splitterDir, -1, wxDefaultPosition, wxDefaultSize, wxCLIP_CHILDREN);
-    m_thumbPanel = new wxPanel(m_splitterThumb, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL|wxCLIP_CHILDREN);
+    m_splitterDir = new wxSplitterWindow(m_splitterTitle, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxCLIP_CHILDREN);
+    m_splitterThumb = new wxSplitterWindow(m_splitterDir, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxCLIP_CHILDREN);
+    m_thumbPanel = new wxPanel(m_splitterThumb, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL|wxCLIP_CHILDREN);
     m_menubar = new wxMenuBar();
     SetMenuBar(m_menubar);
     wxMenu* wxglade_tmp_menu_1 = new wxMenu();
-    wxglade_tmp_menu_1->Append(wxID_NEW, _("&New"), wxT(""), wxITEM_NORMAL);
-    wxglade_tmp_menu_1->Append(wxID_OPEN, _("&Open..."), wxT(""), wxITEM_NORMAL);
-    wxglade_tmp_menu_1->Append(wxID_SAVE, _("&Save"), wxT(""), wxITEM_NORMAL);
-    wxglade_tmp_menu_1->Append(wxID_SAVEAS, _("S&ave as..."), wxT(""), wxITEM_NORMAL);
+    wxglade_tmp_menu_1->Append(wxID_NEW, _("&New"), wxEmptyString, wxITEM_NORMAL);
+    wxglade_tmp_menu_1->Append(wxID_OPEN, _("&Open..."), wxEmptyString, wxITEM_NORMAL);
+    wxglade_tmp_menu_1->Append(wxID_SAVE, _("&Save"), wxEmptyString, wxITEM_NORMAL);
+    wxglade_tmp_menu_1->Append(wxID_SAVEAS, _("S&ave as..."), wxEmptyString, wxITEM_NORMAL);
     wxglade_tmp_menu_1->AppendSeparator();
-    wxglade_tmp_menu_1->Append(MENU_BURN_ID, _("&Burn DVD..."), wxT(""), wxITEM_NORMAL);
+    wxglade_tmp_menu_1->Append(MENU_BURN_ID, _("&Burn DVD..."), wxEmptyString, wxITEM_NORMAL);
     wxglade_tmp_menu_1->AppendSeparator();
-    wxglade_tmp_menu_1->Append(wxID_EXIT, _("&Exit"), wxT(""), wxITEM_NORMAL);
+    wxglade_tmp_menu_1->Append(wxID_EXIT, _("&Exit"), wxEmptyString, wxITEM_NORMAL);
     m_menubar->Append(wxglade_tmp_menu_1, _("&File"));
     wxMenu* wxglade_tmp_menu_2 = new wxMenu();
     wxMenu* wxglade_tmp_menu_2_sub = new wxMenu();
-    wxglade_tmp_menu_2_sub->Append(MENU_DVD_ADD_MENU_ID, _("&Menu"), wxT(""), wxITEM_NORMAL);
-    wxglade_tmp_menu_2_sub->Append(MENU_DVD_ADD_VMMENU_ID, _("&vmMenu"), wxT(""), wxITEM_NORMAL);
-    wxglade_tmp_menu_2_sub->Append(MENU_DVD_ADD_TITLESET_ID, _("&Titleset"), wxT(""), wxITEM_NORMAL);
-    wxglade_tmp_menu_2->Append(wxNewId(), _("&Add"), wxglade_tmp_menu_2_sub, wxT(""));
+    wxglade_tmp_menu_2_sub->Append(MENU_DVD_ADD_MENU_ID, _("&Menu"), wxEmptyString, wxITEM_NORMAL);
+    wxglade_tmp_menu_2_sub->Append(MENU_DVD_ADD_VMMENU_ID, _("&vmMenu"), wxEmptyString, wxITEM_NORMAL);
+    wxglade_tmp_menu_2_sub->Append(MENU_DVD_ADD_TITLESET_ID, _("&Titleset"), wxEmptyString, wxITEM_NORMAL);
+    wxglade_tmp_menu_2->Append(wxID_ANY, _("&Add"), wxglade_tmp_menu_2_sub, wxEmptyString);
     wxglade_tmp_menu_2->AppendSeparator();
-    wxglade_tmp_menu_2->Append(MENU_DVD_OPTIONS_ID, _("&Options..."), wxT(""), wxITEM_NORMAL);
+    wxglade_tmp_menu_2->Append(MENU_DVD_OPTIONS_ID, _("&Options..."), wxEmptyString, wxITEM_NORMAL);
     m_menubar->Append(wxglade_tmp_menu_2, _("&DVD"));
     wxMenu* wxglade_tmp_menu_3 = new wxMenu();
-    wxglade_tmp_menu_3->Append(MENU_SETTINGS_ID, _("&Settings..."), wxT(""), wxITEM_NORMAL);
+    wxglade_tmp_menu_3->Append(MENU_SETTINGS_ID, _("&Settings..."), wxEmptyString, wxITEM_NORMAL);
     m_menubar->Append(wxglade_tmp_menu_3, _("&Configuration"));
     wxMenu* wxglade_tmp_menu_4 = new wxMenu();
-    wxglade_tmp_menu_4->Append(wxID_ABOUT, _("&About..."), wxT(""), wxITEM_NORMAL);
+    wxglade_tmp_menu_4->Append(wxID_ABOUT, _("&About..."), wxEmptyString, wxITEM_NORMAL);
     m_menubar->Append(wxglade_tmp_menu_4, _("&Help"));
     m_statusbar = CreateStatusBar(2, 0);
-    m_dirBar = new wxToolBar(this, -1, wxDefaultPosition, wxDefaultSize, wxNO_BORDER|wxTB_VERTICAL|wxTB_FLAT);
-    m_dirTree = new wxGenericDirCtrl(m_splitterDir, DIR_TREE_ID, wxDirDialogDefaultFolderStr, wxDefaultPosition, wxDefaultSize, wxDIRCTRL_DIR_ONLY );
-    m_thumbLabel = new wxStaticText(m_thumbPanel, -1, _("Directory"), wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE|wxST_NO_AUTORESIZE);
+    m_dirBar = new wxToolBar(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxNO_BORDER|wxTB_VERTICAL|wxTB_FLAT);
+    m_dirTree = new DirCtrl(m_splitterDir, DIR_TREE_ID);
+    m_thumbLabel = new wxStaticText(m_thumbPanel, wxID_ANY, _("Directory"), wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE|wxST_NO_AUTORESIZE);
     m_thumbnails = new wxThumbnails(m_thumbPanel, THUMBNAILS_ID);
     m_menuEditor = new MenuEditor(m_splitterThumb, MENU_EDITOR_ID);
     m_titlesetManager = new TitlesetManager(m_splitterTitle, TITLESET_MANAGER_ID);
@@ -189,7 +189,13 @@ MainWin::MainWin(): wxFrame(NULL, -1, _T(""), wxDefaultPosition, wxDefaultSize,
     
     wxMenu* thumbMenu = new wxMenu;
     thumbMenu->Append(THUMBMENU_BACKGROUND_ID, _("&Assign to background"));
+    thumbMenu->AppendSeparator();
+    thumbMenu->Append(REFRESH_ID, _("&Refresh") + wxString(wxT("\tF5")));
     m_thumbnails->SetPopupMenu(thumbMenu);
+    
+    wxMenu* thumbGlobalMenu = new wxMenu;
+    thumbGlobalMenu->Append(REFRESH_ID, _("&Refresh") + wxString(wxT("\tF5")));
+    m_thumbnails->SetGlobalPopupMenu(thumbGlobalMenu);
 }
 
 void MainWin::set_properties()
@@ -200,19 +206,18 @@ void MainWin::set_properties()
     // begin wxGlade: MainWin::set_properties
     SetTitle(_("DVDStyler"));
     SetSize(wxSize(1000, 740));
-    int m_statusbar_widths[] = { -1, 250 };
+    int m_statusbar_widths[] = { -1, 275 };
     m_statusbar->SetStatusWidths(2, m_statusbar_widths);
     const wxString m_statusbar_fields[] = {
-        wxT(""),
-        wxT("")
+        wxEmptyString,
+        wxEmptyString
     };
     for(int i = 0; i < m_statusbar->GetFieldsCount(); ++i) {
         m_statusbar->SetStatusText(m_statusbar_fields[i], i);
     }
-    m_dirBar->SetSize(wxSize(40, -1));
     m_thumbLabel->SetBackgroundColour(wxColour(90, 16, 198));
     m_thumbLabel->SetForegroundColour(wxColour(255, 255, 255));
-    m_titlesetManager->SetSize(wxSize(955, 124));
+    m_titlesetManager->SetMinSize(wxSize(955, 124));
     // end wxGlade
 	
 	m_thumbLabel->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHT));
@@ -245,7 +250,7 @@ void MainWin::set_properties()
     // dir bar
     m_dirBar->SetMargins(4, 4);
     wxVerticalToolbar* vertToolBar = new wxVerticalToolbar(m_dirBar);
-    vertToolBar->AddTool(DIR_TREE_BT_ID, _("Directories"), wxBITMAP_FROM_MEMORY(files),
+    vertToolBar->AddTool(DIR_TREE_BT_ID, _("File browser"), wxBITMAP_FROM_MEMORY(files),
 	  wxITEM_RADIO);
     vertToolBar->AddTool(BACKGROUNS_BT_ID, _("Backgrounds"), wxNullBitmap,
 	  wxITEM_RADIO);
@@ -377,6 +382,7 @@ void MainWin::OnButtonsBt(wxCommandEvent& event)
   for (int i=0; i<(int)buttons.Count(); i++)
   {
 	wxThumb* thumb = new wxThumb(buttons[i]);
+	wxLogNull log;
 	MenuObject bt(&menu, buttons[i], 0, 0, _("button"));
 	thumb->SetImage(bt.GetImage(m_thumbnails->GetThumbWidth(),
 	  m_thumbnails->GetThumbHeight()));
@@ -403,38 +409,43 @@ void MainWin::OnThumbInfo(wxCommandEvent& event)
 		text = _("Double click to set the background.");
 	else if (m_dirBar->GetToolState(BUTTONS_BT_ID))
 		text = m_thumbnails->GetPointedItem()->GetFilename().AfterLast(wxFILE_SEP_PATH);
-	else
-		text = m_thumbnails->GetPointedItem()->GetFilename();
+	else {
+		double size = wxFile(m_thumbnails->GetPointedItem()->GetFilename()).Length()/1024;
+		if (size < 1024)
+			text.Printf(_("%s (%.1f KB)"),m_thumbnails->GetPointedItem()->GetFilename().c_str(), size);
+		else if (size < 1024*1024)
+			text.Printf(_("%s (%.1f MB)"),m_thumbnails->GetPointedItem()->GetFilename().c_str(), size/1024);
+		else
+			text.Printf(_("%s (%.1f GB)"),m_thumbnails->GetPointedItem()->GetFilename().c_str(), size/1024/1024);
+	}
   }
   m_statusbar->SetStatusText(text, 0);
 }
 
-void MainWin::OnTitleInfo(wxCommandEvent& event)
-{
-  wxString text;
-  if (m_titlesetManager->GetPointed() != -1)
-  {
-	int id = m_titlesetManager->GetPointedId();
-	Vob* vob = m_dvd->GetVob(DVD::GetTsi(id), DVD::IsMenu(id),
-	  DVD::GetPgci(id), DVD::GetVobi(id));
-	if (vob)
-	{
-	  text = vob->GetFilename();
-      for (unsigned int i=0; i<vob->GetAudioFilenames().Count(); i++)
-        text += _T(" + ") + vob->GetAudioFilenames()[i];
-      for (unsigned int i=0; i<vob->GetSubtitles().Count(); i++)
-        text += _T(" + ") + vob->GetSubtitles()[i]->GetFilename();
-	  if (text.length())
-	  {
-		int size = vob->GetSize();
-		if (size>=1024)
-		  text += wxString::Format(_(" (%dMB)"), size/1024);
-		else if (size > 0)
-		  text += wxString::Format(_(" (%dKB)"), size);
-	  }
+void MainWin::OnTitleInfo(wxCommandEvent& event) {
+	wxString text;
+	if (m_titlesetManager->GetPointed() != -1) {
+		int id = m_titlesetManager->GetPointedId();
+		Vob* vob = m_dvd->GetVob(DVD::GetTsi(id), DVD::IsMenu(id),
+				DVD::GetPgci(id), DVD::GetVobi(id));
+		if (vob) {
+			text = vob->GetFilename();
+			for (unsigned int i=0; i<vob->GetAudioFilenames().Count(); i++)
+				text += (text.length() ? wxT(" + ") : wxT("Video + ")) + vob->GetAudioFilenames()[i];
+			for (unsigned int i=0; i<vob->GetSubtitles().Count(); i++)
+				text += wxT(" + ") + vob->GetSubtitles()[i]->GetFilename();
+			if (text.length()) {
+				double size = vob->GetSize();
+				if (size>=1024*1024)
+					text += wxString::Format(_(" (%.1f GB)"), size/1024/1024);
+				else if (size>=1024)
+					text += wxString::Format(_(" (%.1f MB)"), size/1024);
+				else if (size > 0)
+					text += wxString::Format(_(" (%.1f KB)"), size);
+			}
+		}
 	}
-  }
-  m_statusbar->SetStatusText(text, 0);
+	m_statusbar->SetStatusText(text, 0);
 }
 
 void MainWin::OnMenuObjectInfo(wxCommandEvent& event)
@@ -470,6 +481,10 @@ void MainWin::OnSetBackground(wxCommandEvent& event)
   m_menuEditor->SetBackground(filename);
 }
 
+void MainWin::OnRefresh(wxCommandEvent& event) {
+	m_thumbnails->UpdateItems();
+}
+
 void MainWin::OnUpdateUISetBackground(wxUpdateUIEvent& event)
 {
   if (m_thumbnails->GetSelected() == -1)
@@ -479,7 +494,7 @@ void MainWin::OnUpdateUISetBackground(wxUpdateUIEvent& event)
     wxString filename = m_thumbnails->GetSelectedItem()->GetFilename();
     event.Enable(
       wxImage::FindHandler(filename.AfterLast(wxT('.')).Lower(), -1) ||
-      MPEG::IsValid(filename));
+      wxThumbnails::IsVideo(filename));
   }
 }
 
@@ -673,50 +688,63 @@ void MainWin::OnSaveAs(wxCommandEvent& event)
   else
 	wxLogError(_("Nothing to save"));
 }
-
-void MainWin::OnBurn(wxCommandEvent& event)
-{
-  int vobcnt = 0;
-  for (int tsi = 0; tsi<(int)m_dvd->GetTitlesets().Count(); tsi++)
-  {
-	Titleset* ts = m_dvd->GetTitlesets()[tsi];
-	for (int pgci = 0; pgci<(int)ts->GetTitles().Count(); pgci++)
-	{
-	  if (ts->GetTitles()[pgci]->GetVobs().Count()>0)
-	  {
-		vobcnt = 1;
-		break;
-	  }
+/**
+ * Starts compiling without closing
+ */
+void MainWin::OnBurn(wxCommandEvent& event) {
+	Burn(false);
+}
+/**
+ * Opens the burn dialog
+ */
+void MainWin::Burn(bool autoStart) {
+	// check if every titleset contains titles
+	bool hasVobs = false;
+	int emptyTsi = -1;
+	for (int tsi = 0; tsi<(int)m_dvd->GetTitlesets().Count(); tsi++) {
+		Titleset* ts = m_dvd->GetTitlesets()[tsi];
+		bool tsHasVobs = false;
+		for (int pgci = 0; pgci<(int)ts->GetTitles().Count(); pgci++) {
+			if (ts->GetTitles()[pgci]->GetVobs().Count()>0) {
+				hasVobs = true;
+				tsHasVobs = true;
+				break;
+			}
+		}
+		if (!tsHasVobs)
+			emptyTsi = tsi;
 	}
-  }
-  // check if user added some titles
-  if (vobcnt == 0) {
-	wxMessageBox(
-	  _("There are no titles specified. To add a title please drag\n a video file into the bottom section of the window."),
-	  _("Burn"), wxOK|wxICON_ERROR, this);
-	return;
-  }
-  // check if the last title doesn't contain only video
-  PgcArray& pgcs = m_dvd->GetTitlesets().Last()->GetTitles();
-  Pgc* pgc = pgcs.Count()>0 ? pgcs.Last() : NULL;
-  Vob* vob = pgc && pgc->GetVobs().Count()>0 ? pgc->GetVobs().Last() : NULL;
-  if (vob && vob->GetFilename().length() && !vob->HasAudio()) {
-	  wxMessageBox(wxString(_("The last title isn't complete.\n")) +
-			  _("Please add an audio track to it."),
-			  _("Burn"), wxOK|wxICON_ERROR, this);
-	  return;
-  }
-  
-  BurnDlg burnDlg(this);
-  if (burnDlg.ShowModal() != wxID_OK)
-	return;
-  
-  // Progress dialog
-  ProgressDlg progressDlg(this);
-  progressDlg.Start(&burnDlg, m_dvd);
-  
-  DonateDlg donateDlg(this);
-  donateDlg.ShowModal();
+	if (!hasVobs) {
+		wxMessageBox(_("Project doesn't contain any video title. To add a title please\n\
+drag a video file into the bottom section of the window."), _("Burn"), wxOK|wxICON_ERROR, this);
+		return;
+	}
+	if (emptyTsi >= 0) {
+		wxMessageBox(wxString::Format(_("Titleset %d doesn't contain any video title. To add a title\n\
+please drag a video file into the bottom section of the window."), emptyTsi+1), _("Burn"), wxOK|wxICON_ERROR, this);
+		return;
+	}
+	// check if the last title doesn't contain only video
+	PgcArray& pgcs = m_dvd->GetTitlesets().Last()->GetTitles();
+	Pgc* pgc = pgcs.Count()>0 ? pgcs.Last() : NULL;
+	Vob* vob = pgc && pgc->GetVobs().Count()>0 ? pgc->GetVobs().Last() : NULL;
+	if (vob && vob->GetFilename().length() && !vob->HasAudio()) {
+		wxMessageBox(wxString(_("The last title isn't complete.\n")) + _("Please add an audio track to it."), 
+		_("Burn"), wxOK|wxICON_ERROR, this);
+		return;
+	}
+	// show burn dialog ()
+	BurnDlg burnDlg(this);
+	if (!autoStart && burnDlg.ShowModal() != wxID_OK)
+		return;
+	// Progress dialog
+	ProgressDlg progressDlg(this, autoStart);
+	progressDlg.Start(&burnDlg, m_dvd);
+	if (!autoStart) {
+		DonateDlg donateDlg(this);
+		donateDlg.ShowModal();
+	} else
+		Close(true);
 }
 
 void MainWin::OnExit(wxCommandEvent& event)
