@@ -3,7 +3,7 @@
 // Purpose:     The widget to edit a DVD Menu
 // Author:      Alex Thuering
 // Created:     11.10.2003
-// RCS-ID:      $Id: MenuEditor.cpp,v 1.20 2008/04/18 20:17:25 ntalex Exp $
+// RCS-ID:      $Id: MenuEditor.cpp,v 1.22 2008/10/26 20:05:31 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -33,74 +33,68 @@
 ////////////////////////////// MenuDnDFile ///////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
 
-class MenuDnDFile : public wxFileDropTarget
-{
-  public:
-    MenuDnDFile(MenuEditor* menuEditor) { m_menuEditor = menuEditor; }
+class MenuDnDFile : public wxFileDropTarget {
+public:
+	MenuDnDFile(MenuEditor* menuEditor) {
+		m_menuEditor = menuEditor;
+	}
 
-    virtual bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames)
-    {
-      if (!m_menuEditor->GetMenu())
-        return false;
-	  bool res = false;
-	  for (int i = 0; i<(int)filenames.Count(); i++)
-	  {
-		res = AddFile(x, y, filenames[i]) || res;
-		if (x < m_menuEditor->GetMenu()->GetResolution().x-48 &&
-			y < m_menuEditor->GetMenu()->GetResolution().y-48)
-		{
-		  x += 16;
-		  y += 16;
+	virtual bool OnDropFiles(wxCoord x, wxCoord y,
+			const wxArrayString& filenames) {
+		if (!m_menuEditor->GetMenu())
+			return false;
+		bool res = false;
+		for (int i = 0; i<(int)filenames.Count(); i++) {
+			res = AddFile(x, y, filenames[i]) || res;
+			if (x < m_menuEditor->GetMenu()->GetResolution().x-48
+					&& y < m_menuEditor->GetMenu()->GetResolution().y-48) {
+				x += 16;
+				y += 16;
+			}
 		}
-	  }
-	  return res;
-    }
-	
-    bool AddFile(wxCoord x, wxCoord y, wxString filename)
-    {
-      wxString objId = m_menuEditor->HitTest(x, y);
-      x = (int) (x/m_menuEditor->GetScale());
-      y = (int) (y/m_menuEditor->GetScale());
-      wxString ext = filename.AfterLast('.').Lower();
-      if (ext == _T("xml"))
-        m_menuEditor->AddButton(filename, x, y);
-      else if (wxImage::FindHandler(ext, -1))
-        m_menuEditor->AddImage(filename, x, y);
-      else  if (wxThumbnails::IsVideo(filename))
-      {
-        if (!MPEG::IsValid(filename))
-        {
-          wxLogError(filename + _(": not valid mpeg-file"));
-          return false;
-        }
-        if (m_menuEditor->GetTitlesetManager()->AddVideo(filename))
-        {
-          int title = m_menuEditor->GetDVD()->GetPgcArray(m_menuEditor->GetTsi(), false).GetCount()-1;
-          if (objId.length() && m_menuEditor->GetMenu()->GetObject(objId)->IsButton())
-          {
-            MenuObject* obj = m_menuEditor->GetMenu()->GetObject(objId);
-            MenuObjectParam* initParam = obj->GetInitParam();
-            if (initParam && initParam->type == wxT("image"))
-            {
-               Vob* vob =  m_menuEditor->GetDVD()->GetVob(m_menuEditor->GetTsi(), false, title, 0);
-               if (vob)
-               {
-                 obj->SetParam(initParam->name, vob->GetVideoFrameUri(0));
-                 m_menuEditor->Select(objId);
-               }
-            }
-            obj->SetActionPgci(title*2+1);
-          }
-          else
-            m_menuEditor->AddButton(BUTTONS_DIR + wxT("frame.xml"), x, y, title);
-        }
-      }
-      m_menuEditor->SetFocus();
-      return true;
-    }
-	
-  protected:
-    MenuEditor *m_menuEditor;
+		return res;
+	}
+
+	bool AddFile(wxCoord x, wxCoord y, wxString filename) {
+		wxString objId = m_menuEditor->HitTest(x, y);
+		x = (int) (x/m_menuEditor->GetScale());
+		y = (int) (y/m_menuEditor->GetScale());
+		wxString ext = filename.AfterLast('.').Lower();
+		if (ext == _T("xml"))
+			m_menuEditor->AddButton(filename, x, y);
+		else if (wxImage::FindHandler(ext, -1))
+			m_menuEditor->AddImage(filename, x, y);
+		else if (wxThumbnails::IsVideo(filename)) {
+			wxFfmpegMediaDecoder decoder;
+			if (!decoder.Load(filename)) {
+				wxLogError(filename + _(": unknown format"));
+				return false;
+			}
+			if (m_menuEditor->GetTitlesetManager()->AddVideo(filename)) {
+				int title = m_menuEditor->GetDVD()->GetPgcArray(m_menuEditor->GetTsi(), false).GetCount()-1;
+				if (objId.length() && m_menuEditor->GetMenu()->GetObject(objId)->IsButton()) {
+					MenuObject* obj = m_menuEditor->GetMenu()->GetObject(objId);
+					MenuObjectParam* initParam = obj->GetInitParam();
+					if (initParam && initParam->type == wxT("image")) {
+						Vob* vob = m_menuEditor->GetDVD()->GetVob(m_menuEditor->GetTsi(), false, title,
+								0);
+						if (vob) {
+							obj->SetParam(initParam->name,
+									vob->GetVideoFrameUri(0));
+							m_menuEditor->Select(objId);
+						}
+					}
+					obj->SetActionPgci(title*2+1);
+				} else
+					m_menuEditor->AddButton(BUTTONS_DIR + wxT("frame.xml"), x, y, title);
+			}
+		}
+		m_menuEditor->SetFocus();
+		return true;
+	}
+
+private:
+	MenuEditor *m_menuEditor;
 };
 
 //////////////////////////////////////////////////////////////////////////////
@@ -186,8 +180,7 @@ BEGIN_EVENT_TABLE(MenuEditor, wxSVGCtrl)
   EVT_MENU(MENUEDITOR_GRIDMENU_ITEM_ID+5, MenuEditor::OnGridMenu)
   EVT_MENU(MENUEDITOR_GRIDMENU_SHOW_ID, MenuEditor::OnShowGrid)
   EVT_MENU(MENUEDITOR_GRIDMENU_OVEROBJECTS_ID, MenuEditor::OnShowGridOverObjects)
-  EVT_MENU_RANGE(MENUEDITOR_ADDMENU_OBJECT_ID, MENUEDITOR_ADDMENU_OBJECT_ID+999,
-                 MenuEditor::OnAddObject)
+  EVT_MENU_RANGE(MENUEDITOR_ADDMENU_OBJECT_ID, MENUEDITOR_ADDMENU_OBJECT_ID+999, MenuEditor::OnAddObject)
 END_EVENT_TABLE()
 
 DEFINE_EVENT_TYPE(EVT_COMMAND_MENUEDITOR_MENU_CHANGED)
@@ -288,30 +281,28 @@ void MenuEditor::SetMenu(DVD* dvd, Menu* menu, int tsi, int pgci)
   DoSelect();
 }
 
-bool MenuEditor::SetBackground(wxString fname)
-{
-  if (!m_menu)
-	return false;
-  
-  if (wxImage::FindHandler(fname.AfterLast(wxT('.')).Lower(), -1))
-  {
-    wxImage img;
-	if (!img.LoadFile(fname))
-	  return false;
-  }
-  else if (!MPEG::IsValid(fname))
-  {
-	wxLogError(fname + _(": unknown or not supported format"));
-	return false;
-  }
-  
-  if (m_menu->GetBackground() != fname)
-  {
-    m_menu->SetBackground(fname);
-    Update();
-  }
-  
-  return true;
+bool MenuEditor::SetBackground(wxString fname) {
+	if (!m_menu)
+		return false;
+
+	if (wxImage::FindHandler(fname.AfterLast(wxT('.')).Lower(), -1)) {
+		wxImage img;
+		if (!img.LoadFile(fname))
+			return false;
+	} else if (wxThumbnails::IsVideo(fname)) {
+		wxFfmpegMediaDecoder decoder;
+		if (!decoder.Load(fname)) {
+			wxLogError(fname + _(": unknown format"));
+			return false;
+		}
+	}
+
+	if (m_menu->GetBackground() != fname) {
+		m_menu->SetBackground(fname);
+		Update();
+	}
+
+	return true;
 }
 
 void MenuEditor::SetBackgroundColour(wxColour colour)
@@ -409,7 +400,7 @@ bool MenuEditor::AddObject(wxString fname, int x, int y)
         _("Input text"), _T("Text"), this);
     else if (initParam->type == wxT("image"))
       param = wxFileSelector(_("Please choose an image to insert"),
-        wxT(""), wxGetWorkingDirectory(), wxT(""),
+        wxT(""), wxGetCwd(), wxT(""),
         _("Image Files ") + 
 #if wxCHECK_VERSION(2,6,0)
         wxImage::GetImageExtWildcard(),
@@ -503,6 +494,7 @@ void MenuEditor::DoSelect(wxString id, bool refresh, bool sendEvent)
 
 void MenuEditor::OnMouseLeftButton(wxMouseEvent &event)
 {
+  SetFocus();
   if (!m_menu)
 	return;
   int x = event.GetX();
@@ -571,6 +563,7 @@ void MenuEditor::OnMouseMove(wxMouseEvent &event)
 
 void MenuEditor::OnMouseRightButton(wxMouseEvent &event)
 {
+  SetFocus();
   if (event.RightDown() || !m_menu)
 	return;
   // select Object
