@@ -3,7 +3,7 @@
 // Purpose:     DVD menu button properties dialog
 // Author:      Alex Thuering
 // Created:	20.11.2003
-// RCS-ID:      $Id: MenuObjectPropDlg.cpp,v 1.18 2008/08/24 13:25:24 ntalex Exp $
+// RCS-ID:      $Id: MenuObjectPropDlg.cpp,v 1.19 2008/10/25 21:02:48 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -71,7 +71,7 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer)
     m_chapterChoice = (wxChoice*) m_controls[m_controls.Count()-1];
     UpdateChapters();
     jumpSizer->Add(16,8);
-    AddCheckProp(jumpSizer, _("Play All"), m_object->IsPlayAll());
+    AddCheckProp(jumpSizer, _("Play all titles"), m_object->IsPlayAll());
     grid->Add(jumpSizer);
     
     wxString customAction = m_object->GetCustomAction();
