@@ -3,7 +3,7 @@
 // Purpose:     Progress of generation/burning dialog
 // Author:      Alex Thuering
 // Created:     14.08.2004
-// RCS-ID:      $Id: ProgressDlg.cpp,v 1.60 2008/05/18 19:17:02 ntalex Exp $
+// RCS-ID:      $Id: ProgressDlg.cpp,v 1.65 2008/11/09 20:00:20 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -11,6 +11,9 @@
 #include "ProgressDlg.h"
 #include "Config.h"
 #include "MPEG.h"
+#include "mediaenc_ffmpeg.h"
+#include "mediatrc_ffmpeg.h"
+#include "Version.h"
 #include <wxVillaLib/PipeExecute.h>
 #include <wxVillaLib/utils.h>
 #include <wx/dir.h>
@@ -19,8 +22,6 @@
 #include <wx/regex.h>
 #include <wxSVG/SVGDocument.h>
 #include <wxSVG/mediadec_ffmpeg.h>
-#include "mediaenc_ffmpeg.h"
-#include "mediatrc_ffmpeg.h"
 
 #define DATA_FILE(fname) wxFindDataFile(_T("data") + wxString(wxFILE_SEP_PATH) + fname)
 
@@ -226,16 +227,16 @@ BEGIN_EVENT_TABLE(ProgressDlg, wxDialog)
   EVT_BUTTON(ICONIZE_BT_ID, ProgressDlg::OnMinimize)
 END_EVENT_TABLE()
 
-ProgressDlg::ProgressDlg(wxWindow* parent):
+ProgressDlg::ProgressDlg(wxWindow* parent, bool autoStart):
   wxDialog(parent, -1, wxEmptyString, wxDefaultPosition, wxDefaultSize,
   wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxTHICK_FRAME)
 {
     // begin wxGlade: ProgressDlg::ProgressDlg
-    m_summaryLabel = new wxStaticText(this, -1, _("Summary:"));
-    m_summaryText = new wxTextCtrl(this, -1, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxTE_RICH);
-    m_gauge = new wxGauge(this, -1, 10, wxDefaultPosition, wxDefaultSize, wxGA_HORIZONTAL|wxGA_PROGRESSBAR|wxGA_SMOOTH);
-    m_detailsLabel = new wxStaticText(this, -1, _("Details:"));
-    m_detailsText = new wxTextCtrl(this, -1, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxTE_RICH);
+    m_summaryLabel = new wxStaticText(this, wxID_ANY, _("Summary:"));
+    m_summaryText = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxTE_RICH);
+    m_gauge = new wxGauge(this, wxID_ANY, 10, wxDefaultPosition, wxDefaultSize, wxGA_HORIZONTAL|wxGA_SMOOTH);
+    m_detailsLabel = new wxStaticText(this, wxID_ANY, _("Details:"));
+    m_detailsText = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxTE_RICH);
     m_detailsBt = new wxButton(this, HIDE_BT_ID, _("Hide details"));
     m_minimizeBt = new wxButton(this, ICONIZE_BT_ID, _("Minimize"));
     m_cancelBt = new wxButton(this, wxID_CANCEL, _("Cancel"));
@@ -245,6 +246,7 @@ ProgressDlg::ProgressDlg(wxWindow* parent):
     // end wxGlade
     m_cancel = false;
     m_subStepCount = 0;
+    m_autoStart = autoStart;
 }
 
 void ProgressDlg::set_properties()
@@ -275,7 +277,6 @@ void ProgressDlg::do_layout()
     btSizer->Add(m_minimizeBt, 0, wxRIGHT|wxALIGN_CENTER_VERTICAL, 8);
     btSizer->Add(m_cancelBt, 0, 0, 0);
     mainSizer->Add(btSizer, 0, wxALL|wxEXPAND, 10);
-    SetAutoLayout(true);
     SetSizer(mainSizer);
     Layout();
     Centre();
@@ -342,6 +343,8 @@ void ProgressDlg::AddDetailText(const wxString& text)
 {
   m_detailsText->AppendText(text);
   m_detailsText->ShowPosition(m_detailsText->GetLastPosition());
+  if (wxLog::GetActiveTarget()->GetVerbose())
+	fprintf(stderr, "%s", (const char*) text.mb_str());
 }
 
 void ProgressDlg::UpdateGauge()
@@ -399,7 +402,11 @@ void ProgressDlg::Start(BurnDlg* burnDlg, DVD* dvd) {
 	delete wxLog::SetActiveTarget(previousLog);
 	delete m_winDisabler;
 	m_winDisabler = NULL;
-	ShowModal();
+	// close the Window or release the controls
+	if (m_autoStart)
+		Close(true);
+    else
+    	ShowModal();
 }
 
 void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd)
@@ -407,6 +414,10 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd)
   if (IsCanceled())
 	return;
   
+  // print version
+  AddDetailMsg(wxT("DVDStyler v") + APP_VERSION);
+  AddDetailMsg(wxFfmpegMediaEncoder::GetBackendVersion());
+  
   wxString tmpDir = burnDlg->GetTempDir();
   if (tmpDir.Last() != wxFILE_SEP_PATH)
     tmpDir += wxFILE_SEP_PATH;
@@ -427,13 +438,13 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd)
 	  Vob* vob = pgc->GetVobs()[0];
 	  vob->SetTmpFilename(menuFile);
 	  menuSubSteps += 10; // perpare
-	  if (vob->GetMenu()->HasVideoBackground())
-	  {
-		if (!MPEG::HasNavPacks(vob->GetMenu()->GetBackground()))
-		  menuSubSteps += 200; // transcode(200)
+	  if (vob->GetMenu()->HasVideoBackground()) {
+		  menuSubSteps += 200; // transcode
 	  }
+	  else if (vob->GetAudioFilenames().size() > 0)
+		menuSubSteps += 300; // generate mpeg(100) + transcode(200)
 	  else
-        menuSubSteps += 50; // generate mpeg
+        menuSubSteps += 25; // generate mpeg
       menuSubSteps += 50; // spumux
 	  menuVobs.Add(vob);
 	}
@@ -539,7 +550,7 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd)
   AddDetailMsg(_("Cleaning temporary directory"));
   if (wxDir::Exists(tmpDir) && wxFindFirstFile(tmpDir + _T("*"), wxDIR|wxFILE).length())
   {
-    if (wxMessageBox(_("The temporary directory isn't empty.\nThis directory must be cleaned.\nContinue?"),
+    if (wxMessageBox(_("The temporary directory isn't empty. This directory must be cleaned. Continue?"),
 	    _("Burn"), wxYES_NO|wxICON_QUESTION, this) != wxYES)
 	{
 	  AddSummaryMsg(_("Aborted"), wxEmptyString, *wxRED);
@@ -807,19 +818,20 @@ or open the following directory with your DVD player: ");
     }
 	// get disc size
 	long discSize = 0;
-	for (unsigned int i = 0; i < output.Count(); i++)
+	for (unsigned int i = 0; i < output.Count(); i++) {
       if (output[i].length() > 12 && output[i].SubString(1,12) == wxT("Free Blocks:")) {
         wxString discSizeStr = output[i].AfterFirst(wxT(':')).Trim(false).BeforeFirst(wxT('*'));
         discSizeStr.ToLong(&discSize);
         AddDetailMsg(wxString::Format(wxT("Disc size: %d MB"), discSize/512));
         break;
       }
+	}
 	if (discSize < 2290000)
 	  discSize = 2295104;
 	// check size
 	long size = 0;
 	if (burnDlg->DoAddECC()) {
-      size = wxFile(tmpDir + wxT("dvd.iso")).Length();
+      size = wxFile(tmpDir + wxT("dvd.iso")).Length() / 2048; // size in 2048 blocks
     } else {
       cmd = s_config.GetIsoSizeCmd();
       cmd.Replace(_T("$DIR"), tmpDVDDir.Mid(0, tmpDVDDir.length()-1));
@@ -831,11 +843,11 @@ or open the following directory with your DVD player: ");
       wxExecute(cmd, output, wxEXEC_SYNC | wxEXEC_NODISABLE);
       if (output.Count() > 0 && output[0].length() > 0) {
         output[0].ToLong(&size);
-        size = (size + 254) * 2048;
+        size = size + 254;
       }
     }
-    AddDetailMsg(wxString::Format(wxT("ISO Size: %d MB"), size/1024/1024));
-    if (size > discSize*2048
+    AddDetailMsg(wxString::Format(wxT("ISO Size: %d MB"), size/512));
+    if (size > discSize
         && wxMessageBox(wxString::Format(
           _("Size of Disc Image > %.2f GB. Do you want to continue?"), (double)discSize/512/1024),
           _("Burn"), wxYES_NO|wxICON_QUESTION, this) == wxNO)
@@ -892,36 +904,18 @@ or open the following directory with your DVD player: ");
   s_config.SetKey(s_config.GetKey()+1);
 }
 
-bool ProgressDlg::MakeVob(const wxString& videoFile,
-  const wxArrayString& audioFiles, const wxString& vobFile)
-{
-  if (IsCanceled())
-	return false;
-  AddSummaryMsg(wxString::Format(_("Make Vob-file from '%s' and '%s'"),
-	videoFile.c_str(), audioFiles[0].c_str()));
-  // remove old
-  wxString cmd = s_config.GetMplexCmd();
-  cmd.Replace(_T("$FILE_VIDEO"), videoFile);
-  wxString audio;
-  for (unsigned int i = 0; i<audioFiles.Count(); i++)
-    audio += (i>0 ? wxT("\" \"") : wxT(""))  + audioFiles[i];
-  cmd.Replace(_T("$FILE_AUDIO"), audio);
-  cmd.Replace(_T("$FILE_OUT"), vobFile);
-  if (!Exec(cmd))
-  {
-	FailedExec(cmd);
-	return false;
-  }
-  return true;
-}
-
 bool ProgressDlg::Transcode(const StreamArray& streams, const wxString& mpegFile,
 		const wxArrayString& audioFiles, const wxString& vobFile) {
 	if (IsCanceled())
 		return false;
-	AddDetailMsg(_("Fix Video-file: ") + mpegFile);
+	int subStep = GetSubStep();
+	AddDetailMsg(_("Transcode video file: ") + mpegFile);
+	bool disableAudio = streams.GetCount() > 1 + audioFiles.GetCount()
+		&& streams[1]->GetDestinationFormat() == afNONE;
+	if (disableAudio)
+		wxLogError(wxT("disableAudio"));
 	wxFfmpegMediaTranscoder transcoder;
-	if (!transcoder.AddInputFile(mpegFile)) {
+	if (!transcoder.AddInputFile(mpegFile, false, disableAudio)) {
 		Failed(wxT("Error by transcoding of ") + mpegFile);
 		return false;
 	}
@@ -951,19 +945,20 @@ bool ProgressDlg::Transcode(const StreamArray& streams, const wxString& mpegFile
 			break;
 		}
 	}
-	if (!transcoder.SetOutputFile(vobFile, videoFormat, audioFormats, subtitleFormats)
+	if (!transcoder.SetOutputFile(vobFile, videoFormat, audioFormats, subtitleFormats,
+			s_config.GetVideoBitrate(), s_config.GetVbr(), s_config.GetAudioBitrate())
 			|| !transcoder.Run(m_cancel)) {
 		Failed(wxT("Error by transcoding of ") + mpegFile);
 		return false;
 	}
 	transcoder.End();
-	IncSubStep(200);
+	SetSubStep(subStep+200);
 	return true;
 }
 
 bool ProgressDlg::GenerateMenu(Menu* menu, const wxString& menuFile,
 		AudioFormat audioFormat, wxString audioFile) {
-  if (IsCanceled())
+	if (IsCanceled())
 		return false;
 	wxString mpegFile = menuFile + _T("_bg.mpg");
 	wxString m2vFile = menuFile + _T("_bg.m2v");
@@ -971,9 +966,7 @@ bool ProgressDlg::GenerateMenu(Menu* menu, const wxString& menuFile,
 	wxString hlFile = menuFile + _T("_highlight.png");
 	wxString selFile = menuFile + _T("_select.png");
 	wxString spuFile = menuFile + _T("_spumux.xml");
-	wxString silenceFile = audioFormat == afMP2 ? 
-	DATA_FILE(_T("silence.mp2")) : DATA_FILE(_T("silence.ac3"));
-
+	
 	bool videoMenu = menu->HasVideoBackground();
 	wxYield();
 
@@ -1004,14 +997,28 @@ bool ProgressDlg::GenerateMenu(Menu* menu, const wxString& menuFile,
 	
   if (videoMenu) {
 		if (audioFile.length() || !MPEG::HasNavPacks(menu->GetBackground())) {
-			int subStep = GetSubStep();
-			StreamArray streams;
-			wxArrayString audioFiles;
+			Vob menuVob;
+			menuVob.SetFilename(menu->GetBackground());
 			if (audioFile.length())
-				audioFiles.Add(audioFile);
-			if (!Transcode(streams, menu->GetBackground(), audioFiles, mpegFile))
+				menuVob.AddAudioFile(audioFile);
+			for (unsigned int i=0; i<menuVob.GetStreams().GetCount(); i++) {
+				Stream* stream = menuVob.GetStreams()[i];
+				if (stream->GetType() == stVIDEO) {
+					// set destination format (default COPY) if codec is not mpeg2
+					if (stream->GetSourceCodecName() != wxT("mpeg2video"))
+						stream->SetDestinationFormat(menu->GetVideoFormat());
+				} else if (stream->GetType() == stAUDIO) {
+					// set destination format (default COPY) if codec is not mp2/ac3 or sample rate is not 48 kHz
+					if (stream->GetSourceCodecName() != wxT("mp2")
+							&& stream->GetSourceCodecName() != wxT("liba52")) {
+						stream->SetDestinationFormat(audioFormat);
+					} else if (stream->GetSourceSampleRate() != 48000) {
+						stream->SetDestinationFormat(stream->GetSourceCodecName() == wxT("mp2") ? afMP2 : afAC3);
+					}
+				}
+			}
+			if (!Transcode(menuVob.GetStreams(), menuVob.GetFilename(), menuVob.GetAudioFilenames(), mpegFile))
 				return false;
-			SetSubStep(subStep+200);
 		} else
 			mpegFile = menu->GetBackground();
 	} else {
@@ -1028,11 +1035,10 @@ bool ProgressDlg::GenerateMenu(Menu* menu, const wxString& menuFile,
 						frameCount = (int) (duration * 30000/1001);
 				}
 			}
-		} else
-			audioFile = silenceFile;
+		} 
 		wxFfmpegMediaEncoder ffmpeg;
 		if (!ffmpeg.BeginEncode(audioFile.length() ? m2vFile : mpegFile, menu->GetVideoFormat(),
-				audioFile.length() ? afNONE : audioFormat, s_config.GetMenuVideoBitrate())
+				audioFormat, s_config.GetMenuVideoBitrate())
 				|| !ffmpeg.EncodeImage(bgImage, frameCount)) {
 			Failed(_("Error creation of menu"));
 			return false;
@@ -1041,23 +1047,33 @@ bool ProgressDlg::GenerateMenu(Menu* menu, const wxString& menuFile,
 		IncSubStep(25);
 
 		if (audioFile.length()) {
-			// mplex
+			IncSubStep(75);
+			// mplex (and optionally transcode audio)
 			if (IsCanceled())
 				return false;
 			AddDetailMsg(_("Multiplexing audio and video"));
-			wxString cmd = s_config.GetMplexCmd();
-			cmd.Replace(_T("$FILE_VIDEO"), m2vFile);
-			cmd.Replace(_T("$FILE_AUDIO"), audioFile);
-			cmd.Replace(_T("$FILE_OUT"), mpegFile);
-			if (!Exec(cmd)) {
-				FailedExec(cmd);
-				return false;
+			Vob menuVob;
+			menuVob.SetFilename(m2vFile);
+			menuVob.AddAudioFile(audioFile);
+			for (unsigned int i=0; i<menuVob.GetStreams().GetCount(); i++) {
+				Stream* stream = menuVob.GetStreams()[i];
+				if (stream->GetType() == stAUDIO) {
+					// set destination format (default COPY) if codec is not mp2/ac3 or sample rate != 48 kHz
+					if (stream->GetSourceCodecName() != wxT("mp2")
+							&& stream->GetSourceCodecName() != wxT("liba52")) {
+						stream->SetDestinationFormat(audioFormat);
+					} else if (stream->GetSourceSampleRate() != 48000) {
+						stream->SetDestinationFormat(stream->GetSourceCodecName() == wxT("mp2") ? afMP2 : afAC3);
+					}
+				}
 			}
+			menuVob.GetStreams()[1]->SetDestinationFormat(afNONE); // skip generated empty audio stream
+			if (!Transcode(menuVob.GetStreams(), menuVob.GetFilename(), menuVob.GetAudioFilenames(), mpegFile))
+				return false;
 			if (s_config.GetRemoveTempFiles())
 				if (!DeleteFile(m2vFile))
 					return false;
 		}
-		IncSubStep(25);
 	}
 
 	//spumux
