@@ -3,7 +3,7 @@
 // Purpose:     Titleset Manager
 // Author:      Alex Thuering
 // Created:	27.01.2003
-// RCS-ID:      $Id: TitlesetManager.cpp,v 1.18 2008/07/07 18:43:15 ntalex Exp $
+// RCS-ID:      $Id: TitlesetManager.cpp,v 1.20 2008/11/09 19:59:55 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -398,15 +398,19 @@ bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle) {
 	for (unsigned int i=0; i<vob->GetStreams().GetCount(); i++) {
 		Stream* stream = vob->GetStreams()[i];
 		if (stream->GetType() == stVIDEO) {
-			if (stream->GetSourceCodecName() != wxT("mpeg2video"))
+			// set destination format (default COPY) if codec is not mpeg2
+			if (stream->GetSourceCodecName() != wxT("mpeg2video")
+					|| stream->GetSourceVideoFormat() != m_dvd->GetVideoFormat())
 				stream->SetDestinationFormat(m_dvd->GetVideoFormat());
-			else // do not remultiplex/transcode if the video is already a VOB
-				vob->SetDoNotTranscode(vob->GetAudioFilenames().Count() == 0
-						&& MPEG::HasNavPacks(vob->GetFilename()));
+			else if (vob->GetAudioFilenames().Count() == 0 && MPEG::HasNavPacks(vob->GetFilename()))
+				vob->SetDoNotTranscode(true);  // do not remultiplex/transcode if the video is already a VOB
 		} else if (stream->GetType() == stAUDIO) {
-			if (stream->GetSourceCodecName() != wxT("mp2")
-					&& stream->GetSourceCodecName() != wxT("ac3"))
+			// set destination format (default COPY) if codec is not mp2/ac3 or sample rate is not 48 kHz
+			if (stream->GetSourceAudioFormat() != m_dvd->GetAudioFormat()
+					|| stream->GetSourceSampleRate() != 48000) {
 				stream->SetDestinationFormat(m_dvd->GetAudioFormat());
+				vob->SetDoNotTranscode(false);
+			}
 		}
 	}
 	// set chapters
@@ -457,6 +461,14 @@ bool TitlesetManager::AddAudio(const wxString& fname) {
 	Vob* vob = pgcs.Last()->GetVobs().Last();
 	vob->AddAudioFile(fname);
 	vob->SetDoNotTranscode(false);
+	// check if reencoding is needed
+	Stream* stream = vob->GetStreams()[vob->GetStreams().GetCount()-1];
+	if (vob->GetStreams().GetCount() == 2 && stream->GetSourceAudioFormat() != m_dvd->GetAudioFormat())
+		stream->SetDestinationFormat(m_dvd->GetAudioFormat());
+	else if (stream->GetSourceAudioFormat() != afMP2 && stream->GetSourceAudioFormat() != afAC3)
+		stream->SetDestinationFormat(m_dvd->GetAudioFormat());
+	else if (stream->GetSourceSampleRate() != 48000)
+		stream->SetDestinationFormat(stream->GetSourceAudioFormat());
 	UpdateItems();
 	SendDvdChangedEvent();
 	return true;
