@@ -3,7 +3,7 @@
 // Purpose:     Media Transcoder Interface
 // Author:      Alex Thuering
 // Created:     26.04.2008
-// RCS-ID:      $Id: mediatrc.h,v 1.3 2008/05/18 15:59:01 ntalex Exp $
+// RCS-ID:      $Id: mediatrc.h,v 1.4 2008/11/09 19:56:43 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -21,8 +21,11 @@ public:
 	virtual bool AddInputFile(const wxString& fileName, bool disableVideo = false,
 			bool disableAudio = false, bool disableSubtitle = false) = 0;
 	/** Sets output file and video/audio/subtitle formats. */
-	virtual bool SetOutputFile(const wxString& fileName, VideoFormat videoFormat,
-			AudioFormat audioFormat, SubtitleFormat subtitleFormat, int videoBitrate = 6000) = 0;
+	virtual bool SetOutputFile(const wxString& fileName, VideoFormat videoFormat, AudioFormat audioFormat,
+		SubtitleFormat subtitleFormat, int videoBitrate = 6000, bool vbr = false, int audioBitrate = 224) = 0;
+	/** Sets output file and video/audio/subtitle formats. */
+	virtual bool SetOutputFile(const wxString& fileName, VideoFormat videoFormat, wxArrayInt audioFormats,
+		wxArrayInt subtitleFormats, int videoBitrate = 6000, bool vbr = false, int audioBitrate = 224) = 0;
 	/** Starts transcoding. */
 	virtual bool Run(bool& canceled) = 0;
 	/** Ends ttanscoding and closes all fiies. */
