@@ -3,7 +3,7 @@
 // Purpose:     FFMPEG Media Transcoder
 // Author:      Alex Thuering
 // Created:     26.04.2008
-// RCS-ID:      $Id: mediatrc_ffmpeg.cpp,v 1.5 2008/08/24 20:34:02 ntalex Exp $
+// RCS-ID:      $Id: mediatrc_ffmpeg.cpp,v 1.8 2008/11/09 19:56:43 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -300,7 +300,7 @@ bool wxFfmpegMediaTranscoder::AddInputFile(const wxString& fileName,
 			audio_channels = enc->channels;
 			audio_sample_rate = enc->sample_rate;
 			if (disableAudio)
-				ic->streams[i]->discard= AVDISCARD_ALL;
+				ic->streams[i]->discard = AVDISCARD_ALL;
 			break;
 		case CODEC_TYPE_VIDEO:
 			frame_height = enc->height;
@@ -328,7 +328,7 @@ bool wxFfmpegMediaTranscoder::AddInputFile(const wxString& fileName,
 
 			enc->rate_emu = rate_emu;
 			if (disableVideo)
-				ic->streams[i]->discard= AVDISCARD_ALL;
+				ic->streams[i]->discard = AVDISCARD_ALL;
 			break;
 		case CODEC_TYPE_DATA:
 			break;
@@ -361,8 +361,8 @@ bool wxFfmpegMediaTranscoder::AddInputFile(const wxString& fileName,
 }
 
 bool wxFfmpegMediaTranscoder::SetOutputFile(const wxString& fileName,
-		VideoFormat videoFormat, AudioFormat audioFormat,
-		SubtitleFormat subtitleFormat, int videoBitrate) {
+		VideoFormat videoFormat, AudioFormat audioFormat, SubtitleFormat subtitleFormat,
+		int videoBitrate, bool vbr, int audioBitrate) {
 	wxArrayInt audioFormats;
 	audioFormats.Add(audioFormat);
 	wxArrayInt subtitleFormats;
@@ -372,8 +372,8 @@ bool wxFfmpegMediaTranscoder::SetOutputFile(const wxString& fileName,
 }
 
 bool wxFfmpegMediaTranscoder::SetOutputFile(const wxString& fileName,
-		VideoFormat videoFormat, wxArrayInt audioFormats,
-		wxArrayInt subtitleFormats, int videoBitrate) {
+		VideoFormat videoFormat, wxArrayInt audioFormats, wxArrayInt subtitleFormats,
+		int videoBitrate, bool vbr, int audioBitrate) {
 	AVFormatParameters params, *ap = &params;
 
 	AVOutputFormat *outputFormat = guess_format("dvd", NULL, NULL);
@@ -401,14 +401,12 @@ bool wxFfmpegMediaTranscoder::SetOutputFile(const wxString& fileName,
 		for (int si=0; si<(int)ic->nb_streams; si++) {
 			switch (ic->streams[si]->codec->codec_type) {
 			case CODEC_TYPE_VIDEO:
-				if (videoFormat != vfNONE && !AddVideoStream(oc, videoFormat,
-						videoBitrate))
+				if (videoFormat != vfNONE && !AddVideoStream(oc, videoFormat, videoBitrate, vbr))
 					return false;
 				break;
 			case CODEC_TYPE_AUDIO:
-				if (audioFormats.GetCount() > 0 && audioFormats[afIdx]
-						!= afNONE && !AddAudioStream(oc,
-						(AudioFormat) audioFormats[afIdx]))
+				if (audioFormats.GetCount() > 0 && audioFormats[afIdx] != afNONE
+						&& !AddAudioStream(oc, (AudioFormat) audioFormats[afIdx], audioBitrate))
 					return false;
 				if (afIdx < audioFormats.GetCount() - 1)
 					afIdx++;
@@ -508,7 +506,7 @@ void wxFfmpegMediaTranscoder::PrintError(const wxString& msg, int err) {
 }
 
 bool wxFfmpegMediaTranscoder::AddVideoStream(AVFormatContext *oc,
-		VideoFormat videoFormat, int videoBitrate) {
+		VideoFormat videoFormat, int bitrate, bool vbr) {
 	AVStream *st;
 	AVCodecContext *video_enc;
 
@@ -559,14 +557,14 @@ bool wxFfmpegMediaTranscoder::AddVideoStream(AVFormatContext *oc,
 		video_enc->codec_id = codecId;
 		codec = avcodec_find_encoder(codecId);
 
-		av_set_string(video_enc, "gop", ntsc ? "18" : "15");
-		av_set_int(video_enc, "b", videoBitrate * 1024);
-		av_set_string(video_enc, "maxrate", "9000000");
-		av_set_string(video_enc, "minrate", "0"); //1500000;
-		av_set_string(video_enc, "bufsize", "1835008"); //224*1024*8;
+		av_set_int(video_enc, "gop", ntsc ? 15 : 12);
+		av_set_int(video_enc, "b", bitrate * 1000);
+		av_set_int(video_enc, "maxrate", vbr ? 9000000 : bitrate * 1000);
+		av_set_int(video_enc, "minrate", vbr ? 0 : bitrate * 1000);
+		av_set_int(video_enc, "bufsize", 1835008); //224*1024*8
 
-		av_set_string(video_enc, "packetsize", "2048");
-		av_set_string(video_enc, "muxrate", "10080000");
+		av_set_int(video_enc, "packetsize", 2048);
+		av_set_int(video_enc, "muxrate", 10080000);
 
 		video_enc->time_base.den = fps.num;
 		video_enc->time_base.num = fps.den;
@@ -664,8 +662,7 @@ bool wxFfmpegMediaTranscoder::AddVideoStream(AVFormatContext *oc,
 	return true;
 }
 
-bool wxFfmpegMediaTranscoder::AddAudioStream(AVFormatContext *oc,
-		AudioFormat audioFormat) {
+bool wxFfmpegMediaTranscoder::AddAudioStream(AVFormatContext *oc, AudioFormat audioFormat, int bitrate) {
 	AVStream *st;
 	AVCodecContext *audio_enc;
 
@@ -702,11 +699,7 @@ bool wxFfmpegMediaTranscoder::AddAudioStream(AVFormatContext *oc,
 		st->stream_copy = 1;
 		audio_enc->channels = audio_channels;
 	} else {
-		if (audioFormat == afAC3)
-			av_set_string(audio_enc, "ab", "448000");
-		else
-			av_set_string(audio_enc, "ab", "224000");
-
+		av_set_int(audio_enc, "ab", bitrate*1000);
 		audio_enc->codec_id = codecId;
 
 		if (audio_qscale > QSCALE_NONE) {
@@ -787,9 +780,13 @@ bool wxFfmpegMediaTranscoder::Run(bool& canceled) {
 	j = 0;
 	for (i=0; i<nb_input_files; i++) {
 		is = input_files[i];
+		int nbStreams = 0;
+		for (k=0; k<(int)is->nb_streams; k++)
+			if (is->streams[k]->discard != AVDISCARD_ALL)
+				nbStreams++;
 		file_table[i].ist_index = j;
-		file_table[i].nb_streams = is->nb_streams;
-		j += is->nb_streams;
+		file_table[i].nb_streams = nbStreams;
+		j += nbStreams;
 	}
 	nb_istreams = j;
 
@@ -808,6 +805,8 @@ bool wxFfmpegMediaTranscoder::Run(bool& canceled) {
 	for (i=0; i<nb_input_files; i++) {
 		is = input_files[i];
 		for (k=0; k<(int)is->nb_streams; k++) {
+			if (is->streams[k]->discard == AVDISCARD_ALL)
+				continue;
 			ist = ist_table[j++];
 			ist->st = is->streams[k];
 			ist->file_index = i;
@@ -903,8 +902,7 @@ bool wxFfmpegMediaTranscoder::Run(bool& canceled) {
 					found = 0;
 					for (j=0; j<nb_istreams; j++) {
 						ist = ist_table[j];
-						if (ist->discard && ist->st->codec->codec_type
-								== ost->st->codec->codec_type) {
+						if (ist->discard && ist->st->codec->codec_type == ost->st->codec->codec_type) {
 							ost->source_index = j;
 							found = 1;
 							break;
@@ -958,8 +956,8 @@ bool wxFfmpegMediaTranscoder::Run(bool& canceled) {
 
 			if (!codec->codec_tag) {
 				if ( !os->oformat->codec_tag || av_codec_get_id(
-						os->oformat->codec_tag, icodec->codec_tag) > 0
-						|| av_codec_get_tag(os->oformat->codec_tag,
+						(const struct AVCodecTag **) os->oformat->codec_tag, icodec->codec_tag) > 0
+						|| av_codec_get_tag((const struct AVCodecTag **) os->oformat->codec_tag,
 								icodec->codec_id) <= 0)
 					codec->codec_tag = icodec->codec_tag;
 			}
