@@ -3,7 +3,7 @@
 // Purpose:     wxThumbnails class
 // Author:      Alex Thuering
 // Created:		2.02.2003
-// RCS-ID:      $Id: Thumbnails.cpp,v 1.11 2008/07/07 18:41:31 ntalex Exp $
+// RCS-ID:      $Id: Thumbnails.cpp,v 1.12 2008/10/25 19:15:00 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -134,7 +134,7 @@ BEGIN_EVENT_TABLE(wxThumbnails, wxScrolledWindow)
   EVT_MOTION(wxThumbnails::OnMouseMove)
   EVT_LEAVE_WINDOW(wxThumbnails::OnMouseLeave)
   EVT_THUMBNAILS_THUMB_CHANGED(-1, wxThumbnails::OnThumbChanged)
-  EVT_KEY_UP(wxThumbnails::OnKeyUp)
+  EVT_KEY_DOWN(wxThumbnails::OnKeyDown)
 END_EVENT_TABLE()
 
 DEFINE_EVENT_TYPE(EVT_COMMAND_THUMBNAILS_SEL_CHANGED)
@@ -244,7 +244,9 @@ bool wxThumbnails::IsVideo(const wxString& fname)
 	fname.AfterLast('.').Lower() == _T("m1v") ||
 	fname.AfterLast('.').Lower() == _T("m2v") ||
 	fname.AfterLast('.').Lower() == _T("mpv") ||
+	fname.AfterLast('.').Lower() == _T("mp4") ||
 	fname.AfterLast('.').Lower() == _T("avi") ||
+	fname.AfterLast('.').Lower() == _T("wmv") ||
 	fname.AfterLast('.').Lower() == _T("mkv");
 }
 
@@ -628,6 +630,7 @@ void wxThumbnails::OnMouseDown(wxMouseEvent &event)
 	wxCommandEvent evt(EVT_COMMAND_THUMBNAILS_SEL_CHANGED, this->GetId());
     GetEventHandler()->ProcessEvent(evt);
   }
+  SetFocus();
 }
 
 void wxThumbnails::OnMouseUp(wxMouseEvent &event)
@@ -733,7 +736,7 @@ void wxThumbnails::OnThumbChanged(wxCommandEvent &event)
   Refresh();
 }
 
-void wxThumbnails::OnKeyUp(wxKeyEvent& event) {
+void wxThumbnails::OnKeyDown(wxKeyEvent& event) {
 	int oldSelected = m_selected;
 	switch (event.GetKeyCode()) {
 		case WXK_LEFT:
@@ -752,6 +755,9 @@ void wxThumbnails::OnKeyUp(wxKeyEvent& event) {
 			if (m_selected + m_cols <= (int)m_items.GetCount()-1)
 				m_selected += m_cols;
 			break;
+		case WXK_F5:
+			UpdateItems();
+			break;
 		default:
 			event.Skip();
 			break;
